@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/cmd/cmd_display.c,v 1.5 2008/06/05 12:03:31 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/cmd/cmd_display.c,v 1.14 2009/04/09 11:54:07 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -122,9 +122,15 @@ cmd_display_addhdrs(struct session *session,
     else
         template_vals_string(tvals, "$date", "(Unknown date)");
 
-    if (env->subject)
-        template_vals_string(tvals, "$subject", env->subject);
-    else
+    if (env->subject) {
+        char *subject = env->subject;
+
+        subject = (char *)
+            rfc1522_decode(pool_alloc(tvals->pool, strlen(subject)),
+                           strlen(subject), subject, NIL);
+
+        template_vals_string(tvals, "$subject", subject);
+    } else
         template_vals_string(tvals, "$subject", "(No subject)");
 
     return(T);
@@ -174,6 +180,41 @@ cmd_display_addnav(struct session *session,
 
 /* ====================================================================== */
 
+/* Parse RFC 2231 name of form ISO-8859-1''%a3
+ *
+ * Returns simple UTF-8 string or NIL if the input was NIL.
+ */
+
+static char *parse_2231(char *value, struct pool *pool)
+{
+    char *s, *t, *charset;
+
+    if (!value)
+        return(NIL);
+
+    value = pool_strdup(pool, value);
+
+    if ((s=strchr(value, '\'')) && ((t=strchr(s+1, '\'')))) {
+        *s++ = '\0';
+        *t++ = '\0';
+
+        charset = value;
+        value   = t;
+
+        string_url_decode(value);
+
+        if (!strcasecmp(charset, "utf-8"))
+            return(value);
+
+        if (charset[0] == '\0')
+            charset = "ISO-8859-1";   /* Default not specified by RFC 2231 */
+
+        return(utf8_from_string(pool, charset, value, strlen(value)));
+    }
+    return(NIL);
+}
+
+
 /* show_structure():
  *   Recursively render message MIME structure as collection of
  *   nested <ol> with links to body parts.
@@ -202,12 +243,26 @@ show_structure_simple(struct template_vals *tvals,
     name = NIL;
     for (parameter = body->parameter; parameter;
          parameter = parameter->next) {
-        if (!strcasecmp(parameter->attribute, "NAME"))
+        if (!strcasecmp(parameter->attribute, "NAME*")) {
+            name = parse_2231(parameter->value, tvals->pool);
+        } else if (!strcasecmp(parameter->attribute, "NAME")) {
             name = parameter->value;
+        }
     }
     if (!name && body->description)
         name = body->description;
 
+    if (!name && body->disposition.type) {
+        for (parameter = body->disposition.parameter; parameter;
+             parameter = parameter->next) {
+            if (!strcasecmp(parameter->attribute, "FILENAME*")) {
+                name = parse_2231(parameter->value, tvals->pool);
+            } else if (!strcasecmp(parameter->attribute, "FILENAME")) {
+                name = parameter->value;
+            }
+        }
+    }
+
     if (!name)
         name = "";
     template_vals_foreach_string(tvals, "@atts", offset, "name", name);
@@ -321,8 +376,9 @@ show_structure(struct template_vals *tvals, unsigned long *offsetp,
 
 static BOOL
 show_textpart(struct session *session, MAILSTREAM *stream,
-              unsigned long msgno, char *section)
+              unsigned long msgno, char *section, BOOL html_show_images)
 {
+    struct template_vals *tvals = session->template_vals;
     struct options *options = session->options;
     struct prefs *prefs = options->prefs;
     struct request *request = session->request;
@@ -339,7 +395,7 @@ show_textpart(struct session *session, MAILSTREAM *stream,
         return(NIL);
 
     if (body->type != TYPETEXT) {
-        session_message(session, "Invalid body type (should never happen)");
+        session_alert(session, "Invalid body type (should never happen)");
         session_log(session, "show_textpart(): Invalid bodypart");
         return(NIL);
     }
@@ -406,10 +462,20 @@ show_textpart(struct session *session, MAILSTREAM *stream,
         show_html = NIL;
 
     if (show_html) {
+        if (!prefs->html_remote_images) {
+            template_vals_ulong(tvals, "$is_html", 1);
+            if (html_show_images)
+                template_vals_ulong(tvals, "$html_images_shown", 1);
+            template_expand("display_images", tvals, b);
+        }
+
         if (decode_msg == init_msg)
             decode_msg = strdup(init_msg);
-        html_secure(session, b,
+
+        bputs(b, "<div class=\"fix_fonts\">"CRLF);
+        html_secure(session, b, html_show_images,
                     utf8_from_string(pool, charset, decode_msg, len));
+        bputs(b, "</div>"CRLF);
     } else {
         bprintf(b, "<pre>" CRLF);
         wrap_line_html(session, b,
@@ -437,6 +503,28 @@ static STRINGLIST *new_strlst(char **l)
     return (sl);
 }
 
+static int
+find_text_section(BODY *body, BOOL html_inline)
+{
+    PART *part = body->nested.part;
+    int   i = 1, body_plain = 0, body_html = 0;
+
+    for (i = 1; part != NIL; part = part->next, i++) {
+        if (!(body = &part->body))
+            continue;
+
+        if ((body->type != TYPETEXT) || !body->subtype)
+            continue;
+
+        if (!strcasecmp(body->subtype, "plain")) {
+            if (!body_plain) body_plain = i;
+        } else if (!strcasecmp(body->subtype, "html")) {
+            if (!body_html) body_html = i;
+        }
+    }
+    return((body_plain) ? body_plain : body_html);
+}
+
 /* ====================================================================== */
 
 void cmd_display(struct session *session)
@@ -455,6 +543,7 @@ void cmd_display(struct session *session)
     MAILSTREAM *stream = session->stream;
     BODY *body = NIL;
     unsigned long offset = 0;
+    BOOL html_show_images = prefs->html_remote_images;
 
     if (request->method == POST) {
         char *cmd, *folder;
@@ -479,7 +568,7 @@ void cmd_display(struct session *session)
     }
 
     if (!msgmap_update(zm)) {
-        session_message(session, "Failed to update msgmap");
+        session_alert(session, "Failed to update msgmap");
         session_redirect(session, request, "restart");
         return;
     }
@@ -504,6 +593,9 @@ void cmd_display(struct session *session)
     /* Record last command for addressbook take */
     session->take_parent_cmd = "display";
 
+    if ((request->argc == 5) && !strcmp(request->argv[4], "show_images"))
+        html_show_images = T;
+
     section = NIL;
     if (request->argc >= 4)
         section = request->argv[3];
@@ -537,6 +629,16 @@ void cmd_display(struct session *session)
     if (prefs->use_tail_banner)
         template_vals_ulong(tvals, "$use_tail_banner", 1);
 
+    if (prefs->html_inline)
+        template_vals_ulong(tvals, "$html_inline", 1);
+
+    if (prefs->html_remote_images)
+        template_vals_ulong(tvals, "$html_remote_images", 1);
+
+    /* Need different name to avoid collision below */
+    if (section)
+        template_vals_string(tvals, "$section_url", section);
+
     cmd_display_addnav(session, stream, msgno);
 
     if (session->full_hdrs) {
@@ -571,9 +673,18 @@ void cmd_display(struct session *session)
 
     if (section)
         bprintf(b, "<p>Section %s:</p>"CRLF, section);
-    else
+    else if (body->type == TYPEMULTIPART) {
+        int subsection = find_text_section(body, prefs->html_inline);
+
+        if (subsection)
+            section = pool_printf(request->pool, "%d", subsection);
+        else
+            section = "1";
+    } else
         section = "1";
 
+    template_vals_string(tvals, "$section", section);
+
     /* Get body structure for specific message part that we are looking at */
     if ((body = ml_body(session, stream, msgno, section)) == NIL) {
         session_redirect(session, request, "restart");
@@ -612,24 +723,7 @@ void cmd_display(struct session *session)
         bprintf(b, "</pre>" CRLF);
 
     } else if (body->type == TYPEMULTIPART) {
-        PART *part = body->nested.part;
-        int   i = 1, body_plain = 0, body_html = 0, subsection;
-
-        for (i = 1; part != NIL; part = part->next, i++) {
-            if (!(body = &part->body))
-                continue;
-
-            if ((body->type != TYPETEXT) || !body->subtype)
-                continue;
-
-            if (!strcasecmp(body->subtype, "plain")) {
-                if (!body_plain) body_plain = i;
-            } else if (!strcasecmp(body->subtype, "html")) {
-                if (!body_html) body_html = i;
-            }
-        }
-
-        subsection = (body_plain) ? body_plain : body_html;
+        int subsection = find_text_section(body, prefs->html_inline);
 
         if (subsection) {
             if (section)
@@ -637,13 +731,14 @@ void cmd_display(struct session *session)
             else
                 section = pool_printf(pool, "%lu", subsection);
 
-            if (!show_textpart(session, stream, msgno, section)) {
+            if (!show_textpart(session, stream, msgno, section,
+                               html_show_images)) {
                 session_redirect(session, request, "restart");
                 return;
             }
         }
     } else if (body->type == TYPETEXT) {
-        if (!show_textpart(session, stream, msgno, section)) {
+        if (!show_textpart(session, stream, msgno, section, html_show_images)) {
             session_redirect(session, request, "restart");
             return;
         }
