@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/cmd/cmd_forward1.c,v 1.4 2008/07/01 10:27:09 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/cmd/cmd_forward1.c,v 1.6 2008/09/17 17:20:25 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -308,8 +308,8 @@ void cmd_forward1(struct session *session)
     if (draft->have_draft) {
         if (!draft_write(draft)) {
             session_message_clear(session);
-            session_message(session,
-                            "Failed to postpone existing draft before forward");
+            session_alert(session,
+                          "Failed to postpone existing draft before forward");
             session_log(session,
                         "[cmd_forward1] Failed to postponed draft message: %s",
                         ml_errmsg());
@@ -390,8 +390,8 @@ void cmd_forward1(struct session *session)
 
     /* Fetch overview for all messages in folder */
     if (!ml_fetch_overview(session, stream, "1:*", NIL)) {
-        session_message(session, "Failed to fetch folder overview: %s",
-                        ml_errmsg());
+        session_alert(session, "Failed to fetch folder overview: %s",
+                      ml_errmsg());
 
         session_redirect(session, request, "list");
         return;
