@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/lib/os_linux.c,v 1.2 2008/05/19 15:55:56 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/lib/os_linux.c,v 1.4 2009/09/01 10:25:22 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -21,11 +21,9 @@
 
 #include <sys/prctl.h>
 
-#ifdef USE_BSD_PTY
 /* Use BSD terminal stuff: simpler */
 #include <pty.h>                /* for openpty and forkpty */
 #include <utmp.h>               /* for login_tty */
-#endif
 
 /* ====================================================================== */
 
@@ -635,8 +633,7 @@ BOOL os_run(char *cmdline, int *fdp, int *childpidp)
     return (T);
 }
 
-#ifdef USE_BSD_PTY
-/* BSD Psuedo-Terminal support */
+/* Use BSD Psuedo-Terminal support on Linux */
 BOOL os_run_pty(char *cmdline, int *fdp, int *childpidp)
 {
     int fd;
@@ -656,110 +653,4 @@ BOOL os_run_pty(char *cmdline, int *fdp, int *childpidp)
     *childpidp = pid;
     return (T);
 }
-#else
-/* System V Pseudo-Terminal support */
-
-/* PTY handling code inferrred from OpenSSH and Solaris 8 in.telnetd code
- * Conclusion: PTY stuff is awful! */
-
-BOOL os_run_pty(char *cmdline, int *fdp, int *childpidp)
-{
-    extern int grantpt();       /* Provide prototypes for pty fns */
-    extern int unlockpt();      /* Linux hdrs don't define these properly */
-    extern char *ptsname();
-    char *slavename;
-    int masterfd, slavefd;
-    int pid;
-
-    *fdp = (-1);
-    *childpidp = 0;
-
-    masterfd = open("/dev/ptmx", O_RDWR | O_NOCTTY);
-    if (masterfd < 0) {
-        close(masterfd);
-        log_misc("/dev/ptmx: %s", strerror(errno));
-        return (NIL);
-    }
-
-    if (grantpt(masterfd) < 0) {
-        close(masterfd);
-        log_misc("grantpt: %s", strerror(errno));
-        return (NIL);
-    }
-
-    if (unlockpt(masterfd) < 0) {
-        close(masterfd);
-        log_misc("unlockpt: %s", strerror(errno));
-        return (NIL);
-    }
-
-    if ((pid = fork()) < 0)
-        log_fatal("fork() failed");
-
-    if (pid == 0) {
-        /* Child process */
-        slavename = ptsname(masterfd);
-
-        if (slavename == NIL) {
-            close(masterfd);
-            log_misc("Slave pty side name could not be obtained.");
-            return (NIL);
-        }
-
-        /* Put child process in its own session group */
-        /* Removes controlling terminal */
-        (void) setsid();
-
-        /* Open slave side of pty and dup to become stdin, stdout, stderr */
-        /* slavefd becomes new controlling terminal for child process     */
-        if ((slavefd = open(slavename, O_RDWR)) < 0)
-            log_fatal("Couldn't open slave");
-
-        close(masterfd);
-
-        if (isastream(slavefd)) {
-            if (ioctl(slavefd, I_PUSH, "ptem") < 0)
-                log_fatal("ioctl I_PUSH ptem: %s", strerror(errno));
-
-            if (ioctl(slavefd, I_PUSH, "ldterm") < 0)
-                log_fatal("ioctl I_PUSH ldterm: %s", strerror(errno));
-
-            if (ioctl(slavefd, I_PUSH, "ttcompat") < 0)
-                log_fatal("ioctl I_PUSH ttcompat: %s", strerror(errno));
-        }
-#if 0                           /* XXX Testing terminal control: something funny going on here...  */
-        {
-            struct termios attr;
-
-            if (tcgetattr(slavefd, &attr) != 0)
-                log_fatal("tcgetattr() failed");
-
-            attr.c_oflag &= ~(OPOST | IXOFF | IXON);
-            attr.c_lflag &= ~(ISIG | ECHO | XCASE);     /* ICANON? */
-            attr.c_lflag |= ECHONL;
-            attr.c_cflag &= ~(CSIZE | PARENB);
-            attr.c_cflag |= CS8;
-            attr.c_cc[VMIN] = 1;
-            attr.c_cc[VTIME] = 1;
-
-            if (tcsetattr(slavefd, TCSANOW, &attr) != 0)
-                log_fatal("tcgetattr() failed");
-        }
-#endif
-
-        dup2(slavefd, 0);
-        dup2(slavefd, 1);
-        dup2(slavefd, 2);
-        close(slavefd);
-
-        process_exec(cmdline);
-        /* NOTREACHED */
-        exit(1);
-    }
-
-    *fdp = masterfd;
-    *childpidp = pid;
-    return (T);
-}
-#endif
 
