@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/servers/prayer_server.c,v 1.4 2008/05/30 10:04:39 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/servers/prayer_server.c,v 1.10 2009/04/03 13:19:02 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -11,6 +11,43 @@
 #include "prayer_login.h"
 #include "prayer_server.h"
 
+static void
+prayer_compose_dump(struct prayer *prayer, struct request *request, char *user)
+{
+    struct template_vals *tvals = prayer->template_vals;
+    struct buffer *b = request->write_buffer;
+    struct assoc *h  = NIL;
+    char *s;
+
+    request_decode_form(request);
+    h = request->form;
+
+    template_vals_string(tvals, "user", user);
+
+    if ((s=assoc_lookup(h, "hdr_To")))
+        template_vals_string(tvals, "$hdr_to", s);
+
+    if ((s=assoc_lookup(h, "hdr_Cc")))
+        template_vals_string(tvals, "$hdr_cc", s);
+
+    if ((s=assoc_lookup(h, "hdr_Bcc")) && s[0])
+        template_vals_string(tvals, "$hdr_bcc", s);
+ 
+    if ((s=assoc_lookup(h, "hdr_Fcc")) && s[0])
+        template_vals_string(tvals, "$hdr_fcc", s);
+ 
+    if ((s=assoc_lookup(h, "hdr_Reply_To")) && s[0])
+        template_vals_string(tvals, "$hdr_reply_to", s);
+
+    if ((s=assoc_lookup(h, "hdr_Subject")))
+        template_vals_string(tvals, "$hdr_subject", s);
+
+    if ((s=assoc_lookup(h, "body")))
+        template_vals_string(tvals, "body", s);
+ 
+    template_expand("frontend_compose_timeout", tvals, b);
+}
+
 /* check_argv_valid_() ***************************************************
  *
  * Check that the username argument supplied in URL looks sane before
@@ -37,6 +74,27 @@ static BOOL check_argv_valid(char *s)
     return(T);
 }
 
+static void generate_connect_error_page(struct prayer *prayer)
+{
+    struct request *request = prayer->request;
+    struct buffer *b = request->write_buffer;
+    struct template_vals *tvals = prayer->template_vals;
+
+    prayer_template_setup(prayer);
+    tvals = prayer->template_vals;
+
+    if (request->argc >= 2)
+        template_vals_string(tvals, "user", request->argv[1]);
+
+    if ((request->argc >= 5) && (!strcmp(request->argv[4], "compose")) &&
+        (request->method == POST)) {
+        prayer_compose_dump(prayer, request, request->argv[1]);
+    } else {
+        template_expand("frontend_timeout", tvals, b);
+    }
+    response_html(request, 404);
+}
+
 /* prayer_process_session_request() **************************************
  *
  * Process a session URL: connect to session server and proxy request
@@ -110,6 +168,17 @@ static BOOL prayer_process_session_request(struct prayer *prayer)
         }
     }
 
+    /* No session id, even after cookie lookup */
+    if (request->argv[2][0] == '\0') {
+        generate_connect_error_page(prayer);
+
+        /* Do we need to kill session cookie? Next login should kill anyway */
+        request->persist = NIL;
+        response_send(request);
+        prayer_accesslog(prayer, request);
+        return (NIL);
+    }
+
     /* Check decoded string for sneakiness */
     if (!check_argv_valid(request->argv[2])) {
         response_error(request, 404);
@@ -148,21 +217,9 @@ static BOOL prayer_process_session_request(struct prayer *prayer)
     }
 
     if (fd < 0) {
-        struct buffer *b = request->write_buffer;
-        char *url_prefix = prayer_url_prefix(prayer, request->pool);
-
-        html_common_start(config, b, "Session disconnected");
-        bputs(b, "<h3>" CRLF);
-        bputs(b, "Couldn't connect to session. Maybe it timed out?" CRLF);
-        bprintf(b,
-                ("<p>Click <a target=\"_top\" href=\"%s/login/%s\">here</a>"
-                 " to login again</p>"), url_prefix, request->argv[1]);
-        bputs(b, "</h3>" CRLF);
-        html_common_end(b);
+        generate_connect_error_page(prayer);
 
         /* Do we need to kill session cookie? Next login should kill anyway */
-        response_html(request, 404);
-
         request->persist = NIL;
         response_send(request);
         prayer_accesslog(prayer, request);
@@ -282,10 +339,17 @@ prayer_process_request_internal(struct prayer *prayer,
 
     /* Cope with /login/.../xyz screen */
     if ((argc == 2) && (!strcmp(argv[0], "login"))) {
+        string_url_decode(request->argv[1]);
         prayer_login(prayer, request->argv[1]);
         return;
     }
 
+    /* Deal with /raven request */
+    if ((argc == 1) && (!strcmp(argv[0], "raven"))) {
+        prayer_login_raven(prayer);
+        return;
+    }
+
     /* Page not found for anything other than the login page */
     if (argc > 0) {
         response_error(request, 404);
@@ -424,7 +488,6 @@ static BOOL prayer_process_connection(struct prayer *prayer, int sockfd)
     }
     while (persist);
 
-    /* Flush any data in output buffer (should be NOOP) */
     iostream_close(stream);
 
     if (request_count != 1)
@@ -468,8 +531,8 @@ prayer_server_single(struct prayer *prayer, BOOL foreground,
         return (NIL);           /* May be transcient error: continue */
 
     if (foreground) {
+        /* prayer_process_connection() always closes newsockfd */
         prayer_process_connection(prayer, newsockfd);
-        close(newsockfd);
         return (T);
     }
 
@@ -490,10 +553,8 @@ prayer_server_single(struct prayer *prayer, BOOL foreground,
         /* Make sure that RSAkey isn't going to expire immediately */
         iostream_freshen_rsakey(&ssl_config);
 
-        /* Process the connection */
+        /* prayer_process_connection() always closes newsockfd */
         prayer_process_connection(prayer, newsockfd);
-
-        close(newsockfd);
         exit(0);
     }
 
