@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/session/draft.c,v 1.2 2008/05/19 15:55:58 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/session/draft.c,v 1.17 2009/06/30 08:22:32 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -223,27 +223,13 @@ void draft_copy(struct draft *dst, struct draft *src)
 /* draft_clear_hdrs() ****************************************************
  *
  * Clear headers
- *   restores reply to and from personal nameback to default state
+ *   restores reply to to default state
  ************************************************************************/
 
 void draft_clear_hdrs(struct draft *d)
 {
     struct prefs *prefs = d->session->options->prefs;
 
-    if (d->role && d->role->personal && d->role->personal[0])
-        d->from_personal = d->role->personal;
-    else if (prefs->from_personal)
-        d->from_personal = prefs->from_personal;
-    else
-        d->from_personal = NIL;
-
-    if (d->role && d->role->from && d->role->from[0])
-        d->from_address = d->role->from;
-    else if (prefs->from_address)
-        d->from_address = prefs->from_address;
-    else
-        d->from_address = NIL;
-
     d->to = NIL;
     d->cc = NIL;
     d->bcc = NIL;
@@ -316,6 +302,20 @@ void draft_init(struct draft *d)
     d->line_wrap = prefs->line_wrap_on_send;
     d->rich_headers = NIL;
 
+    if (d->role && d->role->personal && d->role->personal[0])
+        d->from_personal = pool_strdup(d->pool, d->role->personal);
+    else if (prefs->from_personal)
+        d->from_personal = pool_strdup(d->pool, prefs->from_personal);
+    else
+        d->from_personal = NIL;
+
+    if (d->role && d->role->from && d->role->from[0])
+        d->from_address = pool_strdup(d->pool, d->role->from);
+    else if (prefs->from_address)
+        d->from_address = pool_strdup(d->pool, prefs->from_address);
+    else
+        d->from_address = NIL;
+
     draft_clear_hdrs(d);
     draft_clear_body(d);
     draft_clear_atts(d);
@@ -491,6 +491,9 @@ void draft_update_body(struct draft *d, struct assoc *h)
     d->pool = pool_create(PREFERRED_DRAFT_BLOCK_SIZE);
 
     /* Duplicate Existing headers into new pool */
+    d->from_personal = pool_maybe_strdup(d->pool, d->from_personal);
+    d->from_address  = pool_maybe_strdup(d->pool, d->from_address);
+
     d->to = pool_maybe_strdup(d->pool, d->to);
     d->cc = pool_maybe_strdup(d->pool, d->cc);
     d->bcc = pool_maybe_strdup(d->pool, d->bcc);
@@ -524,6 +527,9 @@ void draft_update(struct draft *d, struct assoc *h)
 
     d->pool = pool_create(PREFERRED_DRAFT_BLOCK_SIZE);
 
+    d->from_personal = pool_maybe_strdup(d->pool, d->from_personal);
+    d->from_address  = pool_maybe_strdup(d->pool, d->from_address);
+
     if ((s = assoc_lookup(h, "hdr_To")))
         d->to = pool_strdup(d->pool, s);
     else
@@ -807,13 +813,15 @@ void draft_init_rich_headers(struct draft *d)
  *   session:
  *        rb: Output buffer for list of recipients
  *    string: String to expand.
+ *    countp: Running total for number of recipients
  *
  * Returns:   T on sucess
  *          NIL if error parsing header fields
  ************************************************************************/
 
 static BOOL
-draft_make_recip(struct session *session, struct buffer *rb, char *string)
+draft_make_recip(struct session *session, struct buffer *rb,
+                 char *string, unsigned long *countp)
 {
     struct options *options = session->options;
     struct prefs *prefs = options->prefs;
@@ -821,6 +829,7 @@ draft_make_recip(struct session *session, struct buffer *rb, char *string)
     struct abook *abook = session->options->abook;
     ADDRESS *addr = NIL, *a;
     char *text;
+    unsigned long recips = 0;
 
     /* Quietly ignore empty fields */
     if (!(string && string[0]))
@@ -836,8 +845,13 @@ draft_make_recip(struct session *session, struct buffer *rb, char *string)
     }
 
     /* Non empty address */
-    for (a = addr; a; a = a->next)
+    for (a = addr; a; a = a->next) {
         bprintf(rb, " %s@%s", a->mailbox, a->host);
+        recips++;
+    }
+
+    if (countp)
+        (*countp) += recips;
 
     mail_free_address(&addr);
     return (T);
@@ -850,20 +864,26 @@ draft_make_recip(struct session *session, struct buffer *rb, char *string)
  *     draft: Current draft.
  *        rb: Output buffer for list of recipients
  *       len: Length of result returned here if not NIL.
+ *    countp: Number of recipients returned here.
  *
  * Returns: NULL terminated string allocated from session->request->pool
  *          NIL if error parsing header fields
  ************************************************************************/
 
-char *draft_make_recipients(struct draft *draft, unsigned long *len)
+char *draft_make_recipients(struct draft *draft,
+                            unsigned long *len,
+                            unsigned long *countp)
 {
     struct session *session = draft->session;
     struct pool *pool = session->request->pool;
     struct buffer *rb = buffer_create(pool, 4096);
 
-    if (!(draft_make_recip(session, rb, draft->to) &&
-          draft_make_recip(session, rb, draft->cc) &&
-          draft_make_recip(session, rb, draft->bcc)))
+    if (countp)
+        *countp = 0;
+
+    if (!(draft_make_recip(session, rb, draft->to, countp) &&
+          draft_make_recip(session, rb, draft->cc, countp) &&
+          draft_make_recip(session, rb, draft->bcc, countp)))
         return (NIL);
 
     if (len)
@@ -1223,7 +1243,29 @@ static void draft_make_multipart(struct draft *draft, struct buffer *b)
                 type = "application/octet-stream";
 
             if (name && name[0]) {
-                if (strchr(name, ' ') || strchr(name, '\t')) {
+                if (string_has8bit(name) || strchr(name, '"')) {
+                    /* Need RFC 2231 encoding */
+
+                    if (utf8_is_8859_1(name)) {
+                        name=pool_strdup(draft->pool, name);
+                        utf8_to_8859_1(name);
+                        name = string_url_encode(draft->pool, name);
+                        name = pool_printf(draft->pool,
+                                           "ISO-8859-1''%s", name);
+                    } else {
+                        name = string_url_encode(draft->pool, name);
+                        name = pool_printf(draft->pool,
+                                           "UTF-8''%s", name);
+                    }
+
+                    bprintf(b, "Content-Type: %s; name*=%s"CRLF,
+                            type, name);
+                    bprintf(b,
+                            "Content-Disposition: attachment; filename*=%s"CRLF,
+                            name);
+                    bprintf(b, "Content-Description: "CRLF);
+
+                } else if (strchr(name, ' ') || strchr(name, '\t')) {
                     bprintf(b, "Content-Type: %s; name=\"%s\"" CRLF,
                             type, name);
                     bprintf(b,
@@ -1395,14 +1437,19 @@ char *draft_make_msg(struct draft *draft, BOOL postpone,
     char *s, *value;
     char *from_address, *from_personal = NIL;
     BOOL use_sender = NIL;
+    char date[64];
+
+    rfc822_fixed_date(date);        /* Generates fixed length output */
 
     if (draft->from_address && draft->from_address[0]) {
         from_address = draft->from_address;
         use_sender = T;
-    } else
-        from_address
-            =
+    } else if (strchr(session->username, '@')) {
+        from_address = pool_strdup(pool, session->username);
+    } else {
+        from_address =
             pool_printf(pool, "%s@%s", session->username, default_domain);
+    }
 
     if (draft->from_personal && draft->from_personal[0]) {
         from_personal = draft->from_personal;
@@ -1426,14 +1473,32 @@ char *draft_make_msg(struct draft *draft, BOOL postpone,
         }
     }
 
+    /* Order of headers according to the note in section 4.1 of RFC 822 */
+
+    if (!postpone) {
+        bprintf(mb,
+	    "Received: from [%s] by %s" CRLF
+	    "	with HTTP (Prayer-%s); %s" CRLF,
+	    ipaddr_text(session->ipaddr),
+	    config->hostname,
+	    VERSION_PRAYER, date);
+    }
+
+    bprintf(mb, "Date: %s" CRLF, date);
+
     bputs(mb, "From: ");
     draft_make_single_address(session, mb, from_personal, from_address,
                               NIL);
     bputs(mb, "" CRLF);
 
-    if (use_sender)
-        bprintf(mb, "Sender: %s@%s" CRLF, session->username,
-                default_domain);
+    if (use_sender) {
+        if (strchr(session->username, '@')) {
+            bprintf(mb, "Sender: %s" CRLF, session->username);
+        } else {
+            bprintf(mb, "Sender: %s@%s" CRLF, session->username,
+                    default_domain);
+        }
+    }
 
     if (!(draft_make_abook_address(session, mb, "To", draft->to) &&
           draft_make_abook_address(session, mb, "Cc", draft->cc) &&
@@ -1449,15 +1514,6 @@ char *draft_make_msg(struct draft *draft, BOOL postpone,
         bprintf(mb, CRLF);
     }
 
-    bputs(mb, "Date: ");
-    {
-        char date[64];
-
-        rfc822_fixed_date(date);        /* Generates fixed length output */
-        bputs(mb, date);
-    }
-    bputs(mb, "" CRLF);
-
     if (draft->fcc && draft->fcc[0] && !string_filename_valid(draft->fcc)) {
         session_message(session, "Invalid Fcc name");
         return (NIL);
@@ -1478,20 +1534,25 @@ char *draft_make_msg(struct draft *draft, BOOL postpone,
 
             bprintf(mb, "X-Reply-Mbox: %s" CRLF, name);
         }
-    } else {
-        bprintf(mb, "X-Mailer: Prayer v%s" CRLF, VERSION_PRAYER);
-        bprintf(mb, "X-Originating-IP: [%s]" CRLF,
-                ipaddr_text(session->ipaddr));
+
+        /* PINE ignores Reply-To unless the following are present */
+        if ((s = draft->reply_to) && s[0]) {
+            bputs(mb, "X-Our-Headers: Reply-To"CRLF);
+            bputs(mb, "X-Our-ReplyTo: Full"CRLF);
+        }
     }
 
+    bprintf(mb, "Message-ID: %s"CRLF, generate_message_id(config));
+
     if (draft->in_reply_to && draft->in_reply_to[0])
         bprintf(mb, "In-Reply-To: %s"CRLF, draft->in_reply_to);
 
-    bprintf(mb, "Message-ID: %s"CRLF, generate_message_id(config));
-
     if (draft->references && draft->references[0])
         bprintf(mb, "References: %s"CRLF, draft->references);
 
+    if (!postpone)
+        bprintf(mb, "X-Mailer: Prayer v%s" CRLF, VERSION_PRAYER);
+
     bprintf(mb, "Mime-Version: 1.0" CRLF);
 
     if (draft->attlist)
@@ -1670,6 +1731,29 @@ static void draft_parse_reply_uid(struct draft *draft, char *s)
                      "Failed to parse X-Reply-UID headers: %s"), s);
 }
 
+/* ====================================================================== */
+
+/* Extract useful bit from sent-mail name */
+
+#define IMAPURL_PREFIX "imap://"
+
+static char *draft_parse_fcc(char *fcc)
+{
+    char *t;
+
+    /* Match imap://user@imap.hermes.cam.ac.uk/<name> */
+    if (!strncmp(fcc, IMAPURL_PREFIX, strlen(IMAPURL_PREFIX)) &&
+        (t=strchr(fcc+strlen(IMAPURL_PREFIX)+1, '/')))
+        return(t+1);
+
+    /* Match {imap.hermes.cam.ac.uk}<name> */
+    if ((t = strrchr(fcc, '}')) != NIL)
+        return(t+1);
+
+    /* Take anything else as is */
+    return(fcc);
+}
+
 /* draft_restored_postponed() ********************************************
  *
  * Generate draft message from message in postponed folder, including any
@@ -1692,7 +1776,7 @@ draft_restore_postponed(struct draft *d,
     BODY *body = NIL;
     char *init_msg;
     char *decode_msg;
-    char *hdrs, *hdr, *s, *t;
+    char *hdrs, *hdr, *s;
     char *charset = "ISO-8859-1";
     PARAMETER *parameter;
     unsigned long len;
@@ -1721,6 +1805,28 @@ draft_restore_postponed(struct draft *d,
 
     /* Restore hdrs */
 
+    if (env->from) {
+        ADDRESS *a = env->from;
+
+        if (a->mailbox && a->host)
+            d->from_address = pool_printf(d->pool, "%s@%s",
+                                          a->mailbox, a->host);
+
+        if (a->personal) {
+            char *s = pool_strdup(d->pool, a->personal);
+            void *t = pool_alloc(pool, strlen(s));
+
+            d->from_personal
+                = (char *) rfc1522_decode(t, strlen(s), s, NIL);
+        }
+
+        /* XXX Temporary: Just to confirm sensible headers */
+        session_log(session,
+                    ("[draft_restore_postponed] From address: %s <%s>"),
+                    (d->from_personal) ? d->from_personal : "",
+                    (d->from_address) ? d->from_address : "");
+    }
+
     if (env->to) {
         d->to = pool_strdup(d->pool, addr_text(pool, env->to));
 
@@ -1793,10 +1899,9 @@ draft_restore_postponed(struct draft *d,
             while ((*s == ' ') || (*s == '\t'))
                 s++;
 
-            if (!strcasecmp(hdr, "Fcc")) {
-                t = strrchr(s, '}');
-                d->fcc = pool_strdup(d->pool, (t) ? (t+1) : s);
-            } else if (!strcasecmp(hdr, "X-Reply-UID"))
+            if (!strcasecmp(hdr, "Fcc"))
+                d->fcc = pool_strdup(d->pool, draft_parse_fcc(s));
+            else if (!strcasecmp(hdr, "X-Reply-UID"))
                 draft_parse_reply_uid(d, s);
             else if (!strcasecmp(hdr, "In-Reply-To"))
                 d->in_reply_to = pool_strdup(d->pool, s);
