@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/session/html_secure.c,v 1.2 2008/05/19 15:55:58 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/session/html_secure.c,v 1.9 2009/01/18 12:03:15 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -203,8 +203,11 @@ my_strip_entities(char *src)
 /* Trashes input => caller has to generate temporary copy.
  * Better way to do this? */
 
-void html_secure(struct session *session, struct buffer *b, char *input)
+void html_secure(struct session *session, struct buffer *b,
+                 BOOL show_images, char *input)
 {
+    struct options *options = session->options;
+    struct prefs *prefs = options->prefs;
     char *s = input;
     char *tag, *token;
     char c;
@@ -280,10 +283,8 @@ void html_secure(struct session *session, struct buffer *b, char *input)
         case 'I':
             if (MATCH(token, "IFRAME"))
                 strip_tag = T;
-#if 0                           /* Possible that we might want to remove these  */
-            else if (MATCH(token, "IMG"))
+            else if (MATCH(token, "IMG") && !show_images)
                 strip_tag = T;
-#endif
             break;
         case 'H':
             if (MATCH(token, "HTML"))
@@ -325,8 +326,6 @@ void html_secure(struct session *session, struct buffer *b, char *input)
         if (endtag) {
             bputs(b, "</");
             bputs(b, token);
-        } else if (MATCH(token, "A")) {
-            bputs(b, "<a target=\"_blank\"");   /* <a> tags are special case */
         } else {
             bputc(b, '<');
             bputs(b, token);
@@ -335,6 +334,10 @@ void html_secure(struct session *session, struct buffer *b, char *input)
         /* Check each attribute in tag */
         while ((token = html_get_token(&tag))) {
             switch (Utoupper(*token)) {
+            case 'B':
+                if (NMATCH(token, "BACKGROUND=") && !prefs->html_remote_images)
+                    continue;
+                break;
             case 'C':
                 if (NMATCH(token, "CODE="))
                     continue;
@@ -361,15 +364,19 @@ void html_secure(struct session *session, struct buffer *b, char *input)
                     /* Indirect links for HTTP and HTTPS to remove "Referrer:" */
                     my_strip_entities(t);
 
-                    if (NMATCH(t, "http://") || NMATCH(t, "https://")) {
-                        bprintf(b, " TARGET=\"_blank\" HREF=\"%s\"", t);
+                    if (NMATCH(t, "http://") ||
+                        NMATCH(t, "https://")) {
+                        bprintf(b, " target=\"_blank\" href=\"%s\"", t);
                         continue;
-                    } else
-                        if (((NMATCH(t, "ftp:")) || (NMATCH(t, "gopher:"))
-                             || (NMATCH(t, "wais:"))
-                             || (NMATCH(t, "telnet:"))
-                             || (NMATCH(t, "cid:")))) {
-                        bprintf(b, " TARGET=\"_blank\" HREF=\"%s\"", t);
+                    } else if (((NMATCH(t, "ftp:")) ||
+                                (NMATCH(t, "gopher:")) ||
+                                (NMATCH(t, "wais:")) ||
+                                (NMATCH(t, "telnet:")) ||
+                                (NMATCH(t, "cid:")))) {
+                        bprintf(b, " target=\"_blank\" href=\"%s\"", t);
+                        continue;
+                    } else if (NMATCH(t, "#")) {
+                        bprintf(b, " href=\"%s\"", t);
                         continue;
                     } else
                         continue;       /* Remove this HREF tag */
