@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/session/session.c,v 1.3 2008/05/24 08:22:52 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/session/session.c,v 1.10 2009/08/28 15:13:11 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -106,6 +106,9 @@ struct session *session_create(struct config *config)
     session->abook_compose_email = NIL; /* Abook compose option        */
     session->help_enabled = NIL;        /* Help text enabled */
     session->help_toolbar = NIL;        /* Toolbar Help text enabled */
+    session->recips_count_session =  0; /* Running total to spot phishers */
+    session->sending_allow = NIL;       /* Whitelist to override: */
+    session->sending_block = NIL;       /* Shut down compromised accounts */
 
     session->telemetry = NIL;   /* Telemetry information */
     session->telemetry_all = NIL;
@@ -171,10 +174,11 @@ struct session *session_create(struct config *config)
 
     session->zm = msgmap_create(p);     /* Zoommap */
     session->message = memblock_create(p, 64);  /* Feedback message */
+    session->message_isalert = NIL;
     session->options = options_create(config, p);       /* Set up default options */
     session->lookup  = lookup_create(session, p);
                                                 /* List of mailboxes */
-    session->folderlist = folderlist_create(config->maildir, config->hiersep);  
+    session->folderlist = folderlist_create(config->hiersep);  
     session->speller = speller_create(p);       /* Spell check */
     session->draft = draft_create(session);     /* Draft message    */
     session->draft0 = draft_create(session);    /* Previous Draft message */
@@ -343,8 +347,14 @@ session_seed_template(struct session *session, struct template_vals *tvals)
         template_vals_string(tvals, "g_service_name",
                              config->login_service_name);
 
-    if (msg && msg[0])
+    if (config->dualuse)
+        template_vals_ulong(tvals, "g_dualuse", 1);
+
+    if (msg && msg[0]) {
         template_vals_string(tvals, "g_status", msg);
+        if (session->message_isalert)
+            template_vals_ulong(tvals, "g_status_alert", 1);
+    }
 
     session_message_clear(session);
 
@@ -537,20 +547,7 @@ void session_setup_urls(struct session *session)
     }
 
     /* Set up appropriate url_icon_prefix */
-    if (!user_agent->use_short &&
-        (user_agent->use_embed_http && (config->http_icon_port > 0L))) {
-        if (config->http_icon_port != 80)
-            session->url_prefix_icons
-                = pool_printf(p, "http://%s:%d/icons%s",
-                              config->hostname, config->http_icon_port,
-                              icon_opts);
-        else
-            session->url_prefix_icons
-                =
-                pool_printf(p, "http://%s/icons%s", config->hostname,
-                            icon_opts);
-    } else
-        session->url_prefix_icons = pool_printf(p, "/icons%s", icon_opts);
+    session->url_prefix_icons = pool_printf(p, "/icons%s", icon_opts);
 
     quoted_user = string_url_encode(p, session->username);
 
@@ -954,6 +951,7 @@ session_login(struct session * session,
         }
         free(name);
     }
+
     return (T);
 }
 
@@ -1062,9 +1060,6 @@ void session_use_prefs(struct session *session, void *p, BOOL initial)
         user_agent->use_pipelining
             = (ua_initial->use_pipelining && prefs->use_pipelining);
 
-        user_agent->use_embed_http
-            = (ua_initial->use_embed_http && prefs->use_embed_http);
-
         user_agent->use_persist
             = (ua_initial->use_persist && prefs->use_persist);
 
@@ -1073,9 +1068,6 @@ void session_use_prefs(struct session *session, void *p, BOOL initial)
 
         user_agent->use_gzip = (ua_initial->use_gzip && prefs->use_gzip);
 
-        user_agent->is_netscape4
-            = (ua_initial->is_netscape4 && prefs->is_netscape4);
-
     }
 
     /* Setup vanilla user_agent for browser to compare against */
@@ -1387,6 +1379,8 @@ void session_message(struct session *session, char *fmt, ...)
     va_start(ap, fmt);
     memblock_vprintf(m, fmt, ap);
     va_end(ap);
+
+    session->message_isalert = NIL;
 }
 
 /* session_message_clear() ***********************************************
@@ -1399,6 +1393,41 @@ void session_message_clear(struct session *session)
     struct memblock *m = session->message;
 
     mputs(m, "");
+    session->message_isalert = NIL;
+}
+
+/* session_alert() ******************************************************
+ *
+ * Record a status message that will be displayed to user at next
+ * opportunity. Flag this as an "alert" message which will be highlighted.
+ *   session:
+ *       fmt: Format string, followed by additional arguments a la printf
+ *
+ ***********************************************************************/
+
+void session_alert(struct session *session, char *fmt, ...)
+{
+    struct memblock *m = session->message;
+    va_list ap;
+    unsigned long size;
+    char *t;
+
+    if (memblock_size(m) > 0) {
+        if ((t = memblock_data(m)) && (t[0] != '\0'))
+            return;
+    }
+
+    va_start(ap, fmt);
+    size = memblock_vprintf_size(m, fmt, ap);
+    va_end(ap);
+
+    memblock_resize(m, size + 1);
+
+    va_start(ap, fmt);
+    memblock_vprintf(m, fmt, ap);
+    va_end(ap);
+
+    session->message_isalert = T;
 }
 
 /* ====================================================================== */
