@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/shared/config.c,v 1.4 2008/06/24 14:40:49 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/shared/config.c,v 1.9 2008/10/30 11:28:59 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -140,12 +140,13 @@ struct config *config_create(void)
     config->pool = pool;
     config->prefix = NIL;
     config->var_prefix = NIL;
-    config->login_prefix_path = NIL;
-    config->login_suffix_path = NIL;
-    config->session_banner_path = NIL;
+    config->raven_enable = NIL;
+    config->raven_key_path = "";
+    config->login_insert1_path = NIL;
+    config->login_insert2_path = NIL;
+    config->login_template = "login";
     config->motd_path = NIL;
     config->welcome_path = NIL;
-    config->help_dir = NIL;
     config->icon_dir = NIL;
     config->static_dir = NIL;
     config->static_expire_timeout = (7 * 24 * 60 * 60);   /* 7 days  */
@@ -163,13 +164,12 @@ struct config *config_create(void)
     config->check_directory_perms = NIL;
 
     config->log_debug = NIL;
-    config->contact_warning = NIL;
-    config->contact_email = NIL;
     config->fatal_dump_core = NIL;
 
     config->template_path = "../templates";
     config->template_set  = "xhtml_strict";
     config->template_use_compiled = T;
+    config->template_list = NIL;
 
     config->prayer_user = NIL;
     config->prayer_uid = 0;
@@ -187,10 +187,12 @@ struct config *config_create(void)
     config->log_name_nets = NIL;
 
     config->limit_vm = 0;
+    config->recips_max_msg = 0;
+    config->recips_max_session = 0;
+    config->sending_allow_dir = NIL;
+    config->sending_block_dir = NIL;
 
     config->http_cookie_use_port = NIL;
-    config->http_icon_embed = T;
-    config->http_icon_port = 0;
     config->http_max_method_size = 0;
     config->http_max_hdr_size = 0;
     config->http_max_body_size = 0;
@@ -253,6 +255,7 @@ struct config *config_create(void)
     config->accountd_fullname_delay = 0;
     config->vacation_default_days = 31;
     config->accountd_timeout = NIL;
+    config->sieve_maxsize = 0;
     config->sieved_user_map = NIL;
     config->sieved_server = NIL;
     config->sieved_timeout = (9*60);   /* Seconds */
@@ -281,9 +284,11 @@ struct config *config_create(void)
     config->theme_main = NIL;
     config->theme_help = NIL;
 
+    config->allow_raven_login = T;
     config->confirm_expunge = NIL;
     config->confirm_logout = T;
     config->confirm_rm = T;
+    config->expunge_on_exit = NIL;
     config->msgs_per_page = 12;
     config->msgs_per_page_max = 50;
     config->msgs_per_page_min = 4;
@@ -321,19 +326,19 @@ struct config *config_create(void)
     config->use_agg_unmark = T;
     config->use_icons = T;
     config->use_welcome = T;
+    config->use_unread = T;
     config->use_tail_banner = T;
     config->use_mark_persist = NIL;
     config->use_cookie = T;
     config->use_substitution = T;
     config->use_http_1_1 = T;
     config->use_pipelining = T;
-    config->use_embed_http = T;
     config->use_persist = T;
     config->use_short = T;
     config->use_gzip = T;
-    config->is_netscape4 = T;
     config->html_inline = T;
     config->html_inline_auto = T;
+    config->html_remote_images = NIL;
     config->preserve_mimetype = T;
 
     return (config);
@@ -411,6 +416,8 @@ static struct {
     , {
     "accountd_user_map", config_path, OFFSET(accountd_user_map)}
     , {
+    "allow_raven_login", config_bool, OFFSET(allow_raven_login)}
+    , {
     "aspell_path", config_path, OFFSET(aspell_path)}
     , {
     "bin_dir", config_path, OFFSET(bin_dir)}
@@ -425,10 +432,6 @@ static struct {
     , {
     "confirm_rm", config_bool, OFFSET(confirm_rm)}
     , {
-    "contact_email", config_string, OFFSET(contact_email)}
-    , {
-    "contact_warning", config_bool, OFFSET(contact_warning)}
-    , {
     "db_ping_interval", config_time, OFFSET(db_ping_interval)}
     , {
     "default_domain", config_string, OFFSET(default_domain)}
@@ -453,6 +456,8 @@ static struct {
     , {
     "egd_socket", config_path, OFFSET(egd_socket)}
     , {
+    "expunge_on_exit", config_bool, OFFSET(expunge_on_exit)}
+    , {
     "fatal_dump_core", config_bool, OFFSET(fatal_dump_core)}
     , {
     "file_perms", config_perm, OFFSET(file_perms)}
@@ -468,8 +473,6 @@ static struct {
     , {
     "gzip_deny_nets", config_string, OFFSET(gzip_deny_nets)}
     , {
-    "help_dir", config_path, OFFSET(help_dir)}
-    , {
     "hiersep", config_string, OFFSET(hiersep)}
     , {
     "hostname", config_string, OFFSET(hostname)}
@@ -482,14 +485,12 @@ static struct {
     , {
     "html_inline_auto", config_bool, OFFSET(html_inline_auto)}
     , {
+    "html_remote_images", config_bool, OFFSET(html_remote_images)}
+    , {
     "http_cookie_use_port", config_bool, OFFSET(http_cookie_use_port)}
     , {
     "http_dump", config_bool, OFFSET(http_dump)}
     , {
-    "http_icon_embed", config_bool, OFFSET(http_icon_embed)}
-    , {
-    "http_icon_port", config_number, OFFSET(http_icon_port)}
-    , {
     "http_max_body_size", config_number, OFFSET(http_max_body_size)}
     , {
     "http_max_connections", config_number, OFFSET(http_max_connections)}
@@ -518,8 +519,6 @@ static struct {
     , {
     "init_socket_name", config_string, OFFSET(init_socket_name)}
     , {
-    "is_netscape4", config_bool, OFFSET(is_netscape4)}
-    , {
     "ispell_language", config_string, OFFSET(ispell_language)}
     , {
     "ispell_path", config_path, OFFSET(ispell_path)}
@@ -564,11 +563,13 @@ static struct {
     , {
     "login_banner", config_string, OFFSET(login_banner)}
     , {
-    "login_prefix_path", config_path, OFFSET(login_prefix_path)}
+    "login_insert1_path", config_path, OFFSET(login_insert1_path)}
+    , {
+    "login_insert2_path", config_path, OFFSET(login_insert2_path)}
     , {
     "login_service_name", config_string, OFFSET(login_service_name)}
     , {
-    "login_suffix_path", config_path, OFFSET(login_suffix_path)}
+    "login_template", config_string, OFFSET(login_template)}
     , {
     "lookup_fullname", config_path, OFFSET(lookup_fullname)}
     , {
@@ -610,18 +611,28 @@ static struct {
     , {
     "preserve_mimetype", config_bool, OFFSET(preserve_mimetype)}
     , {
+    "raven_enable", config_bool, OFFSET(raven_enable)}
+    , {
+    "raven_key_path", config_path, OFFSET(raven_key_path)}
+    , {
+    "recips_max_msg", config_number, OFFSET(recips_max_msg)}
+    , {
+    "recips_max_session", config_number, OFFSET(recips_max_session)}
+    , {
     "referer_block_invalid", config_bool, OFFSET(referer_block_invalid)}
     , {
     "referer_log_invalid", config_bool, OFFSET(referer_log_invalid)}
     , {
     "return_path_domain", config_string, OFFSET(return_path_domain)}
     , {
+    "sending_allow_dir", config_path, OFFSET(sending_allow_dir)}
+    , {
+    "sending_block_dir", config_path, OFFSET(sending_block_dir)}
+    , {
     "sendmail_path", config_path, OFFSET(sendmail_path)}
     , {
     "sent_mail_folder", config_string, OFFSET(sent_mail_folder)}
     , {
-    "session_banner_path", config_path, OFFSET(session_banner_path)}
-    , {
     "session_idle_time", config_time, OFFSET(session_idle_time)}
     , {
     "session_key_len", config_number, OFFSET(session_key_len)}
@@ -631,6 +642,8 @@ static struct {
     "session_timeout_compose", config_time,
             OFFSET(session_timeout_compose)}
     , {
+    "sieve_maxsize", config_number, OFFSET(sieve_maxsize)}
+    , {
     "sieved_server", config_string, OFFSET(sieved_server)}
     , {
     "sieved_timeout", config_time, OFFSET(sieved_timeout)}
@@ -709,8 +722,6 @@ static struct {
     , {
     "use_cookie", config_bool, OFFSET(use_cookie)}
     , {
-    "use_embed_http", config_bool, OFFSET(use_embed_http)}
-    , {
     "use_gzip", config_bool, OFFSET(use_gzip)}
     , {
     "use_http_1_1", config_bool, OFFSET(use_http_1_1)}
@@ -735,6 +746,8 @@ static struct {
     , {
     "use_tail_banner", config_bool, OFFSET(use_tail_banner)}
     , {
+    "use_unread", config_bool, OFFSET(use_unread)}
+    , {
     "use_welcome", config_bool, OFFSET(use_welcome)}
     , {
     "vacation_default_days", config_number, OFFSET(vacation_default_days)}
@@ -887,6 +900,18 @@ config_clone_language(struct config *config, struct config_language *src)
               src->name);
 }
 
+static void
+config_clone_template(struct config *config, struct config_template *src)
+{
+    struct pool *pool = config->pool;
+    struct config_template *dst =
+        pool_alloc(pool, sizeof(struct config_template));
+
+    dst->description = maybe_pool_strdup(pool, src->description);
+
+    list_push(config->template_list, (struct list_item *) dst, src->name);
+}
+
 static void
 config_clone_theme(struct config *config, struct config_theme *src)
 {
@@ -959,6 +984,7 @@ struct config *config_clone_parsed(struct config *src)
     config->http_port_list = NIL;
     config->local_domain_list = NIL;
     config->ispell_language_list = NIL;
+    config->template_list = NIL;
     config->theme_list = NIL;
 
     /* Clone HTTP Port List */
@@ -985,6 +1011,13 @@ struct config *config_clone_parsed(struct config *src)
     } else
         config->ispell_language_list = NIL;
 
+    /* Clone templates */
+    if (src->template_list) {
+        config->template_list = list_create(config->pool, T);
+        for (li = src->template_list->head; li; li = li->next)
+            config_clone_template(config, (struct config_template *) li);
+    }
+
     /* Clone themes */
     config->theme_main = NIL;
     config->theme_help = NIL;
@@ -1554,6 +1587,56 @@ config_parse_ispell_language(struct config *config,
     return (T);
 }
 
+/* config_parse_template() ***********************************************
+ *
+ * Parse a "template" line
+ *    config:
+ *    lineno:
+ *    option: templaye option to parse
+ ************************************************************************/
+
+static BOOL
+config_parse_template(struct config *config, unsigned long lineno,
+                      char *option)
+{
+    struct pool *pool = config->pool;
+    struct list **listp = &config->template_list;
+    struct config_template *template;
+    char *name;
+    char *desc;
+
+    if ((name = string_get_token(&option)) == NIL) {
+        if (lineno > 0)
+            config_paniclog(config,
+                            "Empty \"template\" clause at line %lu",
+                            lineno);
+        else
+            config_paniclog(config,
+                            "Empty \"template\" clause");
+        return (NIL);
+    }
+    if (!config_parse_string(&desc, option, pool)) {
+        if (lineno > 0)
+            config_paniclog(config,
+                            "Invalid \"template\" clause at line %lu",
+                            lineno);
+        else
+            config_paniclog(config,
+                            "Invalid \"template\" clause");
+        return (NIL);
+    }
+
+    template = pool_alloc(pool, sizeof(struct config_template));
+    template->name = pool_strdup(pool, name);
+    template->description = desc;
+
+    if (*listp == NIL)
+        *listp = list_create(pool, T);
+
+    list_push(*listp, (struct list_item *) template, name);
+    return (T);
+}
+
 /* config_parse_theme() **************************************************
  *
  * Parse an "theme" line
@@ -1741,6 +1824,9 @@ BOOL config_parse_option(struct config * config, char *option)
     if (config_check_keyword(&option, "use_ispell_language"))
         return (config_parse_ispell_language(config, 0L, option));
 
+    if (config_check_keyword(&option, "template"))
+        return (config_parse_template(config, 0L, option));
+
     if (config_check_keyword(&option, "theme"))
         return (config_parse_theme(config, 0L, option));
 
@@ -1950,6 +2036,12 @@ BOOL config_parse_file(struct config * config, char *filename)
             continue;
         }
 
+        if (config_check_keyword(&line, "template")) {
+            if (!config_parse_template(config, cur_lineno, line))
+                okay = NIL;
+            continue;
+        }
+
         if (config_check_keyword(&line, "theme")) {
             if (!config_parse_theme(config, cur_lineno, line))
                 okay = NIL;
@@ -2139,12 +2231,13 @@ BOOL config_check(struct config * config)
     struct config_http_port *chp;
     char hostname[MAX_ADDRESS+1], domainname[MAX_ADDRESS+1];
 
-    TEST_STRING(config->help_dir, "help_dir");
     TEST_STRING(config->icon_dir, "icon_dir");
     TEST_STRING(config->socket_dir, "socket_dir");
     TEST_STRING(config->init_socket_name, "init_socket_name");
     TEST_STRING(config->ssl_session_dir, "ssl_session_dir");
+#ifndef MUTEX_SEMAPHORE
     TEST_STRING(config->lock_dir, "lock_dir");
+#endif
     TEST_STRING(config->log_dir, "log_dir");
     TEST_STRING(config->tmp_dir, "tmp_dir");
     TEST_STRING(config->pid_dir, "pid_dir");
@@ -2216,26 +2309,6 @@ BOOL config_check(struct config * config)
     } else if ((chp = config_find_port(config, T)))
         config->ssl_default_port = chp->port;
 
-    /* Check http_icon_port, derive automatically if not defined */
-    if (config->http_icon_embed) {
-        if (config->http_icon_port) {
-            chp = config_locate_port(config, config->http_icon_port);
-            if (chp == NIL) {
-                config_paniclog(config,
-                                "config: http_icon_port defines unused port: %lu",
-                                config->http_icon_port);
-                return (NIL);
-            }
-            if (chp->use_ssl) {
-                config_paniclog(config,
-                                "config: http_icon_port defines SSL port: %lu",
-                                config->http_icon_port);
-                return (NIL);
-            }
-        } else if ((chp = config_find_port(config, NIL)))
-            config->http_icon_port = chp->port;
-    }
-
     for (li = config->http_port_list->head; li; li = li->next) {
         struct config_http_port *chp = (struct config_http_port *) li;
 
@@ -2575,7 +2648,7 @@ config_check_perms(struct config *config, char *dir, struct stat *sbufp)
     if (!okay) {
         if (config->prayer_user && config->prayer_group)
             log_fatal(("\"%s\" directory must be owned by user \"%s\","
-                       " group \"%s\" and not accessible by others"),
+                       " group \"%s\" and not accessible by othe0rs"),
                       dir, config->prayer_user, config->prayer_group);
         else
             log_fatal(("\"%s\" directory must be owned by uid \"%lu\" and"
@@ -2750,6 +2823,12 @@ BOOL config_mkdirs(struct config * config)
     if (config->pid_dir && config->pid_dir[0])
         config_mkdir(config, tpool, var_prefix, config->pid_dir);
 
+    if (config->sending_allow_dir && config->sending_allow_dir[0])
+        config_mkdir(config, tpool, var_prefix, config->sending_allow_dir);
+
+    if (config->sending_block_dir && config->sending_block_dir[0])
+        config_mkdir(config, tpool, var_prefix, config->sending_block_dir);
+
     pool_free(tpool);
     return (T);
 }
