@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/shared/response.c,v 1.5 2008/06/02 18:12:18 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/shared/response.c,v 1.6 2008/09/16 09:59:58 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -681,8 +681,14 @@ response_file(struct request *request,
             bputs(b, "Content-Type: image/gif" CRLF);
         else if (!strcasecmp(e, ".jpg"))
             bputs(b, "Content-Type: image/jpeg" CRLF);
+        else if (!strcasecmp(e, ".png"))
+            bputs(b, "Content-Type: image/png" CRLF);
+        else if (!strcasecmp(e, ".ico"))
+            bputs(b, "Content-Type: image/png" CRLF);
         else if (!strcasecmp(e, ".css"))
             bputs(b, "Content-Type: text/css" CRLF);
+	else if (!strcasecmp(e, ".js"))
+	    bputs(b, "Content-Type: text/javascript" CRLF);
         else
             bputs(b, "Content-Type: text/plain" CRLF);
     } else
