@@ -1,4 +1,4 @@
-/* $Cambridge: hermes/src/prayer/shared/user_agent.c,v 1.2 2008/05/19 15:55:59 dpc22 Exp $ */
+/* $Cambridge: hermes/src/prayer/shared/user_agent.c,v 1.4 2009/08/20 14:28:23 dpc22 Exp $ */
 /************************************************
  *    Prayer - a Webmail Interface              *
  ************************************************/
@@ -40,11 +40,9 @@ struct user_agent *user_agent_create(struct pool *pool)
     result->use_pipelining = T;
     result->use_http_1_1 = T;
     result->use_pipelining = T;
-    result->use_embed_http = T;
     result->use_persist = T;
     result->use_short = T;
     result->use_gzip = T;
-    result->is_netscape4 = NIL;
 
     return (result);
 }
@@ -81,14 +79,11 @@ static void user_agent_clear(struct user_agent *ua)
     ua->use_substitution = NIL;
     ua->use_direct = NIL;
     ua->use_pipelining = NIL;
-    ua->use_embed_http = NIL;
     ua->use_pipelining = NIL;
     ua->use_http_1_1 = NIL;
-    ua->use_embed_http = NIL;
     ua->use_persist = NIL;
     ua->use_short = NIL;
     ua->use_gzip = NIL;
-    ua->is_netscape4 = NIL;
 }
 
 /* ====================================================================== */
@@ -149,10 +144,6 @@ user_agent_set_option(struct user_agent *ua, char *option, BOOL enable)
         else if (!strcmp(option, "direct"))
             ua->use_direct = enable;
         break;
-    case 'e':
-        if (!strcmp(option, "embed_http"))
-            ua->use_embed_http = enable;
-        break;
     case 'g':
         if (!strcmp(option, "gzip"))
             ua->use_gzip = enable;
@@ -170,9 +161,6 @@ user_agent_set_option(struct user_agent *ua, char *option, BOOL enable)
         if (!strcmp(option, "manual"))
             ua->manual = enable;
         break;
-    case 'n':
-        if (!strcmp(option, "netscape4"))
-            ua->is_netscape4 = enable;
     case 'o':
         if (!strcmp(option, "override"))
             ua->use_override = enable;
@@ -211,7 +199,6 @@ static void user_agent_failsafe(struct user_agent *ua)
     ua->use_direct = NIL;
     ua->use_http_1_1 = NIL;
     ua->use_pipelining = NIL;
-    ua->use_embed_http = NIL;
     ua->use_persist = NIL;
     ua->use_short = NIL;
     ua->use_gzip = NIL;
@@ -265,8 +252,6 @@ void user_agent_generate_form(struct buffer *b, struct user_agent *ua)
     bputs(b, "</tr>" CRLF);
 
     bputs(b, "<tr>" CRLF);
-    html_form_table_checkbox(b, "Enable HTTP inside HTTPS",
-                             "use_embed_http", ua->use_embed_http);
     html_form_table_checkbox(b, "Enable persistent HTTP connections",
                              "use_persist", ua->use_persist);
     bputs(b, "</tr>" CRLF);
@@ -281,8 +266,6 @@ void user_agent_generate_form(struct buffer *b, struct user_agent *ua)
     bputs(b, "<tr>" CRLF);
     html_form_table_checkbox(b, "Enable gzip compression",
                              "use_gzip", ua->use_gzip);
-    html_form_table_checkbox(b, "Treat as Netscape 4",
-                             "is_netscape4", ua->is_netscape4);
     bputs(b, "</tr>" CRLF);
 
     bputs(b, "<tr>" CRLF);
@@ -308,7 +291,6 @@ void user_agent_process_form(struct user_agent *ua, struct assoc *h)
     ua->manual = T;
     ua->use_icons = assoc_lookup(h, "use_icons") ? T : NIL;
     ua->use_cookie = assoc_lookup(h, "use_cookie") ? T : NIL;
-    ua->use_embed_http = assoc_lookup(h, "use_embed_http") ? T : NIL;
     ua->use_pipelining = assoc_lookup(h, "use_pipelining") ? T : NIL;
     ua->use_http_1_1 = assoc_lookup(h, "use_http_1_1") ? T : NIL;
     ua->use_persist = assoc_lookup(h, "use_persist") ? T : NIL;
@@ -317,7 +299,6 @@ void user_agent_process_form(struct user_agent *ua, struct assoc *h)
     ua->use_direct = assoc_lookup(h, "use_direct") ? T : NIL;
     ua->use_override = assoc_lookup(h, "use_override") ? T : NIL;
     ua->use_gzip = assoc_lookup(h, "use_gzip") ? T : NIL;
-    ua->is_netscape4 = assoc_lookup(h, "is_netscape4") ? T : NIL;
 }
 
 
@@ -342,7 +323,6 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
     if (!strncasecmp(browser, "Mozilla/5", strlen("Mozilla/5"))) {
         ua->use_icons = T;
         ua->use_cookie = T;
-        ua->use_embed_http = T;
         ua->use_http_1_1 = T;
         ua->use_pipelining = T;
         ua->use_persist = T;
@@ -360,7 +340,6 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
             /* Opera */
             ua->use_icons = T;
             ua->use_cookie = T;
-            ua->use_embed_http = T;
             ua->use_http_1_1 = NIL;     /* Possible problems with HTTP/1.1? */
             ua->use_pipelining = NIL;
             ua->use_persist = NIL;      /* Opera has problems with KeepAlive stuff */
@@ -369,11 +348,10 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
             /* MSIE */
             ua->use_icons = T;
             ua->use_cookie = T;
-            ua->use_embed_http = T;
             ua->use_http_1_1 = T;       /* Possible problems with HTTP/1.1? */
             ua->use_pipelining = T;
             ua->use_persist = T;
-            ua->use_gzip = T;
+            ua->use_gzip = NIL;
         }
         return;
     }
@@ -383,7 +361,6 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
                      strlen("Mozilla/3.0 (compatible; StarOffice "))) {
         ua->use_icons = T;
         ua->use_cookie = NIL;
-        ua->use_embed_http = NIL;
         ua->use_http_1_1 = NIL;
         ua->use_pipelining = NIL;
         ua->use_persist = T;
@@ -403,22 +380,18 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
         if (s && !strncasecmp(s + 1, "Macintosh", strlen("Macintosh"))) {
             ua->use_icons = T;
             ua->use_cookie = T;
-            ua->use_embed_http = NIL;
             ua->use_http_1_1 = NIL;
             ua->use_pipelining = NIL;
             ua->use_persist = NIL;
             ua->use_gzip = T;
-            ua->is_netscape4 = T;
             return;
         } else if (s && !strncasecmp(s + 1, "Win", strlen("Win"))) {
             ua->use_icons = T;
             ua->use_cookie = T;
-            ua->use_embed_http = NIL;
             ua->use_http_1_1 = T;
             ua->use_pipelining = T;
             ua->use_persist = NIL;
             ua->use_gzip = T;
-            ua->is_netscape4 = T;
             return;
         }
     }
@@ -427,11 +400,9 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
     if (!strncasecmp(browser, "Mozilla/", strlen("Mozilla/"))) {
         ua->use_icons = T;
         ua->use_cookie = T;
-        ua->use_embed_http = NIL;
         ua->use_http_1_1 = T;
         ua->use_pipelining = T;
         ua->use_gzip = T;
-        ua->is_netscape4 = T;
         return;
     }
 
@@ -439,11 +410,9 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
     if (!strncasecmp(browser, "Netscape/", strlen("Netscape/"))) {
         ua->use_icons = T;
         ua->use_cookie = T;
-        ua->use_embed_http = NIL;
         ua->use_http_1_1 = T;
         ua->use_pipelining = T;
         ua->use_gzip = T;
-        ua->is_netscape4 = T;
         return;
     }
 
@@ -451,7 +420,6 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
     if (!strncasecmp(browser, "Lynx/", strlen("Lynx/"))) {
         ua->use_icons = NIL;
         ua->use_cookie = NIL;
-        ua->use_embed_http = NIL;
         ua->use_pipelining = NIL;
         ua->use_gzip = T;
         return;
@@ -461,7 +429,6 @@ void user_agent_setup_browser(struct user_agent *ua, char *browser)
     if (!strncasecmp(browser, "w3m/", strlen("w3m/"))) {
         ua->use_icons = NIL;
         ua->use_cookie = NIL;
-        ua->use_embed_http = NIL;
         ua->use_pipelining = NIL;
         ua->use_gzip = T;
         return;
@@ -631,7 +598,7 @@ void user_agent_parse(struct user_agent *ua, char *string)
  * Convery user agent options into printable representation.
  *      ua: User agent
  *
- * Returns: String containing list of options eg: "cookie,embed_http"
+ * Returns: String containing list of options eg: "cookie"
  ***********************************************************************/
 
 char *user_agent_options(struct user_agent *ua)
@@ -662,9 +629,6 @@ char *user_agent_options(struct user_agent *ua)
     if (ua->use_cookie)
         bputs(b, "cookie,");
 
-    if (ua->use_embed_http)
-        bputs(b, "embed_http,");
-
     if (ua->use_pipelining)
         bputs(b, "pipelining,");
 
@@ -686,9 +650,6 @@ char *user_agent_options(struct user_agent *ua)
     if (ua->use_gzip)
         bputs(b, "gzip,");
 
-    if (ua->is_netscape4)
-        bputs(b, "netscape4,");
-
     if (buffer_size(b) > 0)
         return (buffer_fetch(b, 0, buffer_size(b) - 1, NIL));
     else
